import  sys

folder = str(sys.argv[1])

""" Ignore any logics. """
numOfLogics = 0
currentLogic = ''


""" Use blueJSFile.js, base.dat, and CODE.txt as input.
    Create create.js and WEBSITE.html files. """

readingJSFile = open('engine/scripts/base.js', 'r')
writingJSFile = open('stories/' + folder + '/output/web/js/website.js', 'w')
readingHTMLFile = open('engine/scripts/base.dat', 'r')
writingHTMLFile = open('stories/' + folder + '/output/web/website.html', 'w')
readingLogicFile = open('stories/' + folder + '/code.txt', 'r')


""" This method calls printOutHTML on the images returned by printOutJS. """
def main():
	listOfImages = printOutJS()
	printOutHTML(listOfImages)


""" This method copies lines from base.dat into WEBSITE.html.
    It also adds the images in listOfImages to the <body> of WEBSITE.html. """
def printOutHTML(listOfImages):

	# Save the first line of base.dat in the variable readingLine.
	readingLine = readingHTMLFile.readline().strip()

	# Until the readingLine reaches the <body> of base.dat, copy lines from base.dat to WEBSITE.html.
	while readingLine != "<body>":
		writingHTMLFile.write(readingLine+"\n")
		readingLine = readingHTMLFile.readline().strip()

	# Copy the last line of base.dat up until <body> into WEBSITE.html.
	writingHTMLFile.write(readingLine+"\n")
	readingLine = readingHTMLFile.readline().strip()
	
	# Call addImage on each image in listOfImages, so that each image is added to WEBSITE.html.
	for image in listOfImages:
		writingHTMLFile.write("<img src=\"img/" + image + "\" id=\"" + image + "\" style=\"visibility:hidden;display:none\"></img>\n")

	# Until the readingLine reaches the end of the <html> section, copy the lines of base.dat into WEBSITE.html.
	while readingLine != "</html>":
		writingHTMLFile.write(readingLine+"\n")
		readingLine = readingHTMLFile.readline().strip()

	# Copy the </html> line into WEBSITE.html.
	writingHTMLFile.write(readingLine)


""" This method creates copies base.js into create.js and then creates a function in create.js to
    parse the CODE.txt file and save its image references into listOfImages. """	
# Also returns the list of images that have been added, so the images can be added in the html
def printOutJS():
	listOfImages = []
	sceneNumber = 0
	firstSceneName = ''

	# Copy each line of base.js into create.js.
	for line in readingJSFile:
		writingJSFile.write(line)

	# Define a new function in create.js that is called initStory().
	writingJSFile.write('\nfunction initStory(){\n')

	# Use a while loop to read each line of the file, and handle logic and scene lines separately.
	logicLine = readingLogicFile.readline()
	while logicLine != '':

		# Remove white space from entire line and parse by colons.
		logicLine = logicLine.strip().replace(" ","").replace("	","").split(":")

		# If the logicLine defines a Scene, call addScene, and update the listOfImages, firstSceneName, and sceneNumbers based on the returned values.
		if logicLine[0].upper() == "SCENE" or logicLine[0].upper() == "SS":			
			sceneData = addScene(logicLine, firstSceneName, sceneNumber)
			listOfImages += sceneData[0]
			firstSceneName = sceneData[1]
			sceneNumber = sceneData[2]

		# If the logicLine defines a Logic, call addLogic and add its output to the listOfImages.
		elif logicLine[0].upper() == "LOGIC" or logicLine[0].upper() == "LL":
			listOfImages += addLogic(logicLine, sceneNumber)

		# If the logicLine defines the Grid layout, parse its pieces by equals signs.
		elif logicLine[0].upper() == "GRID" or logicLine[0].upper() == "GG":
			for piece in range(1,len(logicLine)):
				piece = logicLine[piece].split("=")

				# If the compenent is the number of COLUMNS, set numColumns to the value after the equals sign.
				if piece[0].upper() == "COLUMNS" or piece[0].upper() == "C":
					numColumns = piece[1]

				# If the compenent is the number of ROWS, set numRows to the value after the equals sign.
				elif piece[0].upper() == "ROWS" or piece[0].upper() == "R":
					numRows = piece[1]

		# Update logicLine to be the next line of the logic file.	
		logicLine = readingLogicFile.readline()

	# End the initStory() function.
	writingJSFile.write('}')

	# Call SetGrid using the number of rows and columns as parameters.
	writingJSFile.write('SetGrid(' + numColumns +', ' + numRows + ');\n')

	# Return the listOfImages.
	return list(set(listOfImages))


""" This method adds lines of code to the create.js file that store information about a new scene into
	a SceneInfo object instance, and adds it to the mapOfScenes. If the new scene is the the first
	scene, this method will change the current scene and audio to this new scene. """
def addScene(logicLine, firstSceneName, sceneNumber):
	listOfImages = ['']

	# Create a new SceneInfo object instance and set its sceneNumber to the value passed into the method.
	writingJSFile.write('var newScene = new SceneInfo();\n')
	writingJSFile.write('newScene.sceneNumber = ' + str(sceneNumber) + ';\n')

	# Loop through each component in the Scene logic line, and parse each one by equals signs.
	for piece in range(1,len(logicLine)):
		piece = logicLine[piece].split("=")

		# Ignore case of keywords.
		piece[0] = piece[0].upper()

		# If the compenent is a name, set the scene object's name to this name.
		if piece[0] == "NAME" or piece[0] == "N":
			sceneName = piece[1]
			writingJSFile.write('newScene.sceneName = \"' + piece[1] + '\";\n')

		# If the component is a background, set the object's background to this value, and add the image to the listOfImages.
		elif piece[0] == "BACKGROUND" or piece[0] == "B":
			writingJSFile.write('newScene.sceneBackgroundImg = $(\"' + piece[1] + '\");\n')
			listOfImages.append(piece[1])

		# If the compenent is audio, set the object's audio to this value.
		elif piece[0] == "MUSIC" or piece[0] == "M":
			writingJSFile.write('newScene.sceneAudio = \"music/' + piece[1] + '\";\n')

	# If this is the first scene, update firstSceneName accordingly, and make this scene the current scene, and its audio the current audio.
	if sceneNumber == 0:
		firstSceneName = sceneName
		writingJSFile.write('StoryAudio.changeAudio(newScene.sceneAudio);\n')
		writingJSFile.write('DrawingVariables.currentScene = newScene;\n')

	# Add this scene to the mapOfScenes.
	writingJSFile.write('DrawingVariables.mapOfScenes[\"'+sceneName+'\"] = newScene;\n')
	
	# Return information about the scene that future scenes and logics will use.
	return [listOfImages, firstSceneName, (sceneNumber + 1)]


""" This method parses the logicLine and adds its contents to the create.js file, and it adds its images to a listOfImages, which it returns. """	
def addLogic(logicLine, sceneNumber):
	global numOfLogics
	listOfImages = []

	# Name this logic based on how many logics have come before it.
	nameOfLogic = 'logic'+str(numOfLogics)

	# Create a new istance of a StoryLogic object.
	writingJSFile.write('var ' + nameOfLogic +' = new StoryLogic();\n')
	writingJSFile.write(nameOfLogic+'.sceneNumber = ' + str(sceneNumber - 1) + ';\n')

	# Set a fake trigger location in the javascript file.
	writingJSFile.write(nameOfLogic+'.triggerLogicLocation = [0,0];\n')

	# Loops through each component of the logic line, and parse it by equals signs.
	for piece in range(1,len(logicLine)):
		piece = logicLine[piece].split("=")

		# Ingore the case of the keyword.
		piece[0] = piece[0].upper()

		# If the compenent is a name, set the object's logicId to this name, and extract its active state.
		if piece[0] == "NAME" or piece[0] == "N":
			writingJSFile.write(nameOfLogic+'.logicId =\"' + piece[1].split("->")[0] + '\";\n')

			# If the logic is defined as active, set the object's initiallyActive value to true.
			if piece[1].split("->")[1].upper() == "ACTIVE" or piece[1].split("->")[1].upper() == "A":
				writingJSFile.write(nameOfLogic+'.initiallyActive = true;\n')

		# If the compenent is a cause, isolate the image names and coordinates by parsing by commas, parentheses, and arrows.
		elif piece[0] == "CAUSE" or piece[0] == "C":
			for image in piece[1].split("),"):
				image = image.split("->(")
				image[1] = image[1].split(")")[0].split(",")
				# subtract 1 from the coordinates to shift (1,1) to (0,0)
				image = [image[0], str(int(image[1][0])-1), str(int(image[1][1])-1)]

				# Add the image to the listOfImages and to the object's list of images.
				listOfImages.append(image[0])
				writingJSFile.write(nameOfLogic+'.listOfCurrentImages.push(new HiddenImage('+image[1]+','+image[2]+',$(\"'+image[0]+'\")));\n')

		# If the compoent is an effect, isolate the affected logic names and new states, by parsing by commas and arrows.
		elif piece[0] == "EFFECT" or piece[0] == "E":

			# If the affected logic has an arrow to active, add it to the object's list of next logics to turn on, and do the opposite if the arrow is to inactive.
			for effect in piece[1].split(","):
				if effect.split("->")[1].upper() == "ACTIVE" or effect.split("->")[1].upper() == "A":
					writingJSFile.write(nameOfLogic+".listOfNextLogicOn.push(\""+effect.split("->")[0]+"\");\n")
				elif effect.split("->")[1].upper() == "INACTIVE" or effect.split("->")[1].upper() == "I":
					writingJSFile.write(nameOfLogic+".listOfNextLogicOff.push(\""+effect.split("->")[0]+"\");\n")

		# If the component is audio, set the object's audio to this audio file name.
		elif piece[0] == "MUSIC" or piece[0] == "M":
			writingJSFile.write(nameOfLogic+'.audioForLogic = \"music/'+ piece[1] +'\";\n')

		# If the compenent is a scene, set the object's nextScene value to this name.
		elif piece[0] == "SCENE" or piece[0] == "S":
			writingJSFile.write(nameOfLogic+'.nextScene = \"'+ piece[1] + '\";\n')

		# If the compenent is a transition, set the object's transitionImage to this file name, and add it to the listOfImages.
		elif piece[0] == "TRANSITION" or piece[0] == "T":
			writingJSFile.write(nameOfLogic+'.transitionImage = $(\"'+ piece[1].split("->")[0] + '\");\n')
			listOfImages.append(piece[1].split("->")[0])

			# Set the object's transitionDirection to the keyword that the arrow points to.
			d = piece[1].split("->")[1].lower()
			if d == "down" or d == "d":
				writingJSFile.write(nameOfLogic+'.transitionDirection = \"down\";\n')
			elif d == "up" or d == "u":
				writingJSFile.write(nameOfLogic+'.transitionDirection = \"up\";\n')
			elif d == "left" or d == "l":
				writingJSFile.write(nameOfLogic+'.transitionDirection = \"left\";\n')
			elif d == "right" or d == "r":
				writingJSFile.write(nameOfLogic+'.transitionDirection = \"right\";\n')

		# If the component is ordered, then set the object's orderImportant value to true.
		elif piece[0] == "ORDERED" or piece[0] == "O":
			writingJSFile.write(nameOfLogic+'.orderImportant = true;\n')

		# If the component is dependent, then separate the logics by parsing by commas, and add these lgoics to the object's list of dependent logics.
		elif piece[0] == "DEPENDENT" or piece[0] == "D":
			for requiredLogic in piece[1].split(","):
 				writingJSFile.write(nameOfLogic+".listOfDependentLogic.push(\""+requiredLogic+"\");\n")

	# Write a create.js line "push" this logic to the global list of logics.
	writingJSFile.write("ListOfLogics.push("+nameOfLogic+");\n")

	# Increase by 1 the numOfLogics variable, and return the listOfImages generated by that logicLine.
	numOfLogics += 1
	return listOfImages


""" Call the main method (i.e. create the js and html files). """
main()
